; asm.asm
; Assorted assembly routines
; (C) 2000 David Given

include	geos.def
include	geode.def
include	resource.def
include	ec.def
include driver.def
include system.def
include heap.def
include internal/interrupt.def
UseLib internal/streamdr.def
UseLib ui.def
.ioenable

global SYSCALL:far
global LOCKCODE:far
global UNLOCKCODE:far
global SHOWREGISTERS:far
global TERMINATEPROCESS:far

MyGlobals struct
	oldvector fptr
	streamdriver word
	listblock word
	rsrclist dword
	proclist dword
	seglist dword
	procblock word
	rsrcblock word
	console word
	shellpid word
MyGlobals end

ProcES struct
	magic dword
	obj dword
	r_ax word
	r_bx word
	r_cx word
	r_dx word
	r_si word
	r_di word
	r_bp word
	r_ip word
	r_sp word
	r_cs word
	r_ds word
	r_flags word
	returncs word
	returnip word
	returnss word
	returnsp word
	returnbp word
ProcES end
	
global _globals:MyGlobals

; =============================================================================
;                                 C STUB ROUTINES
; =============================================================================

STUBS_TEXT segment

; --- StreamSetNotifyMsg ------------------------------------------------------

SetGeosConvention
global MY_STREAMSETNOTIFYMSG:far
MY_STREAMSETNOTIFYMSG proc far	\
				_driver:word, \
				_stream:word, \
				_event:word, \
				_destination:optr, \
				_message:word
			uses si, di, ds
			.enter
			mov bx, _driver
			call GeodeInfoDriver
			mov ax, _event
			mov bx, _stream
			movdw cxdx, _destination, di
			mov di, DR_STREAM_SET_NOTIFY
			push bp
			mov bp, _message
			call [si.DIS_strategy]
			pop bp
			.leave
			ret
MY_STREAMSETNOTIFYMSG endp

; --- SysSetBIOSInterrupt -----------------------------------------------------

SetGeosConvention
global MY_SYSSETBIOSINTERRUPT:far
MY_SYSSETBIOSINTERRUPT proc far \
				_irq:word, \
				_handler:fptr
			uses ds
			.enter
			call SysLockBIOS
			mov ax, _irq
			movdw dsdx, _handler
			mov ah, 25h
			int 21h
			call SysUnlockBIOS
			.leave
			ret
MY_SYSSETBIOSINTERRUPT endp

; --- SysGetBIOSInterrupt -----------------------------------------------------

SetGeosConvention
global MY_SYSGETBIOSINTERRUPT:far
MY_SYSGETBIOSINTERRUPT proc far	\
				_irq:word
			uses es
			.enter
			call SysLockBIOS
			mov ax, _irq
			mov ah, 35h
			int 21h
			movdw dxax, esbx
			call SysUnlockBIOS
			.leave
			ret
MY_SYSGETBIOSINTERRUPT endp

; --- StartProcess ------------------------------------------------------------

SetGeosConvention
global STARTPROCESS:far
STARTPROCESS proc far \
				_segment:word
			.enter
			; This must be atomic.
			INT_OFF
			mov es, _segment
			mov ax, [es:r_ax]
			mov bx, [es:r_bx]
			mov cx, [es:r_cx]
			mov dx, [es:r_ds]
			mov si, [es:r_si]
			mov di, [es:r_di]
			mov bp, [es:r_bp]
			mov ds, [es:r_ds]
			; Stack change
			mov ss, [es:r_ds]
			mov sp, [es:r_sp]
			; End of stack change
			push [es:r_flags]
			push [es:r_cs]
			push [es:r_ip]
			iret
			.fall_thru
STARTPROCESS endp

; --- ProcessThread -----------------------------------------------------------

SetGeosConvention
global _ProcessThread:far
_ProcessThread proc far \
				_memhandle:word
			.enter
			mov bx, _memhandle
			call MemLock
			mov es, ax
			push es
			push es
			call LOCKCODE
			pop es
			
			push ds
			mov ds, es:[r_ds]
			mov es:[returncs], cs
			mov ax, offset threadexit
			mov es:[returnip], ax
			mov es:[returnss], ss
			mov es:[returnsp], sp
			mov es:[returnbp], bp
			
			INT_OFF
			mov ax, es:[r_ax]
			mov bx, es:[r_bx]
			mov cx, es:[r_cx]
			mov dx, es:[r_dx]
			mov si, es:[r_si]
			mov di, es:[r_di]
			mov bp, es:[r_bp]
			mov ss, es:[r_ds]
			mov sp, es:[r_sp]
			INT_ON

			push es:[r_cs]
			push es:[r_ip]
			retf
			
threadexit:
			INT_OFF
			mov ss, es:[returnss]
			mov sp, es:[returnsp]
			mov bp, es:[returnbp]
			pop ds
			INT_ON
			
			push es
			push es
			call UNLOCKCODE
			pop es

			push es
			push es
			call TERMINATEPROCESS
			pop es
						
			mov bx, _memhandle
			call MemUnlock
			mov bx, _memhandle
			call MemFree
			.leave
			ret
_ProcessThread endp

; --- Show registers ----------------------------------------------------------

ShowRegisters proc far
			.enter
			push ax
			push bx
			push cx
			push dx
			push si
			push di
			push bp
			push cs
			push ds
			push ss
			push es
			push sp
			call SHOWREGISTERS
			.leave
			ret
ShowRegisters endp

; --- Allocate an object block ------------------------------------------------

SetGeosConvention
global MY_USERALLOCOBJBLOCK:far
MY_USERALLOCOBJBLOCK proc far \
				_geodehandle:word
			.enter
			mov bx, _geodehandle
			call UserAllocObjBlock
			mov ax, bx
			.leave
			ret
MY_USERALLOCOBJBLOCK endp

STUBS_TEXT ends

; =============================================================================
;                                FIXED ROUTINES
; =============================================================================

idata segment

SetGeosConvention
global SYSCALLINTERRUPTHANDLER:far
SYSCALLINTERRUPTHANDLER proc far
			mov [es:r_ax], ax
			mov [es:r_bx], bx
			mov [es:r_cx], cx
			mov [es:r_dx], dx
			mov [es:r_si], si
			mov [es:r_di], di
			mov [es:r_bp], bp
			mov [es:r_ds], ds
			pop [es:r_ip]
			pop [es:r_cs]
			pop [es:r_flags]
			; We've now retracted over the interrupt frame.
			mov [es:r_sp], sp
			; Switch stacks.
			mov ss, [es:returnss]
			mov sp, [es:returnsp]
			; Set flags & ds.
			cld
			mov ax, segment idata
			mov ds, ax
			INT_ON
			; Now safe to call GOC code.
			push es
			call SYSCALL
			; ...and exit.
			mov es, ax
			INT_OFF
			mov ax, [es:r_ax]
			mov bx, [es:r_bx]
			mov cx, [es:r_cx]
			mov dx, [es:r_dx]
			mov si, [es:r_si]
			mov di, [es:r_di]
			mov bp, [es:r_bp]
			mov ds, [es:r_ds]
			; Stack change
			mov ss, [es:r_ds]
			mov sp, [es:r_sp]
			; End of stack change
			push [es:r_flags]
			push [es:r_cs]
			push [es:r_ip]
			iret
SYSCALLINTERRUPTHANDLER endp

idata ends
