/* elksdefs.h
 * Various Linux-86 structures and definitions.
 */

/* struct stat */

struct stat {
	word st_dev;
	word st_ino;
	word st_mode;
	word st_nlink;
	word st_uid;
	word st_gid;
	word st_rdev;
	dword st_size;
	dword st_atime;
	dword st_mtime;
	dword st_ctime;
};

/* File modes */

#define S_IFMT   0170000
#define S_IFSOCK 0140000
#define S_IFLNK	 0120000
#define S_IFREG  0100000
#define S_IFBLK  0060000
#define S_IFDIR  0040000
#define S_IFCHR  0020000
#define S_IFIFO  0010000
#define S_ISUID  0004000
#define S_ISGID  0002000
#define S_ISVTX  0001000

#define S_IRWXU 00700
#define S_IRUSR 00400
#define S_IWUSR 00200
#define S_IXUSR 00100

#define S_IRWXG 00070
#define S_IRGRP 00040
#define S_IWGRP 00020
#define S_IXGRP 00010

#define S_IRWXO 00007
#define S_IROTH 00004
#define S_IWOTH 00002
#define S_IXOTH 00001

#define S_IRWXUGO	(S_IRWXU|S_IRWXG|S_IRWXO)
#define S_IALLUGO	(S_ISUID|S_ISGID|S_ISVTX|S_IRWXUGO)
#define S_IRUGO		(S_IRUSR|S_IRGRP|S_IROTH)
#define S_IWUGO		(S_IWUSR|S_IWGRP|S_IWOTH)
#define S_IXUGO		(S_IXUSR|S_IXGRP|S_IXOTH)

#define R_OK		4	/* test for read permission */
#define W_OK		2	/* test for write permission */
#define X_OK		1	/* test for execute permission */
#define F_OK		0	/* test for existence */

/* Open flags */

#define O_ACCMODE	  0003
#define O_RDONLY	    00
#define O_WRONLY	    01
#define O_RDWR		    02
#define O_CREAT		  0100	/* not fcntl */
#define O_EXCL		  0200	/* not fcntl */
#define O_NOCTTY	  0400	/* not fcntl */
#define O_TRUNC		 01000	/* not fcntl */
#define O_APPEND	 02000
#define O_NONBLOCK	 04000
#define O_NDELAY	O_NONBLOCK
#define O_SYNC		010000	/* Not supported */
#define FASYNC		020000	/* Not supported */

/* struct dirent */

struct dirent {
	dword d_ino;
	dword d_off;
	int d_reclen;
	char d_name[256];
};

/* Seek targets */

#define SEEK_SET 0
#define SEEK_CUR 1
#define SEEK_END 2

/* fcntl commands */

#define F_DUPFD		0
#define F_GETFD		1
#define F_SETFD		2
#define F_GETFL		3
#define F_SETFL		4
#define F_GETLK		5
#define F_SETLK		6
#define F_SETLKW	7

/* times() structure */

struct tms {
	long tms_utime;
	long tms_stime;
	long tms_cutime;
	long tms_cstime;
};
